define(
    ['Global', 'User', 'Resources'],
    function (Global, user, resources) {
        'use strict';
        var WaitListEntryEWL = Backbone.Model.extend({
            defaults: {
                "clinicId": "",
                "clinicName": "",
                "dateLastUpdated": "",
                "daysOnList": "",
                "desiredDate": "",
                "dfn": "",
                "enrolleeStatus": "",
                "ewlIen": "",
                "object-type": "WaitListEntry",
                "originatingDate": "",
                "patientName": "",
                "patientZip": "",
                "priority": "",
                "scCategory": "",
                "serviceConnected": "",
                "ssn": "",
                "stopCode": "",
                "waitListType": "EWL"
            },

            initialize: function () {
            },

            convertPercentage: function (pct) {
                return pct >= 50 ? "50 +" : pct < 50 && pct > 0 ? "50 -" : pct !== "" ? "NO" : "";
            },

            parse: function (response) {
                var facilityCode = user.toJSON()['vistaLocation'],
                    patientLink = resources.get('system-root').get('href') + "patient/dfn-" + facilityCode + "/" + response.dfn;

                response.scCategory = this.convertPercentage(response.serviceConnected || "NO");
                response.type = response.waitListType;
                response.originatingDate = formatDate(new Date(response.originatingDate));
                response.appointmentRequestDate = formatDate(new Date(response.appointmentRequestDate));
                response.dateLastUpdated = formatDate(new Date(response.dateLastUpdated));
                response.desiredDate = formatDate(new Date(response.desiredDate)) || response.recallDate || response.appointmentRequestDate;
                response.waitListId = this.getWaitlistId(response);
                response.detailsLink = response.waitListId;
                response.patientLink = patientLink;

                return response;
            },

            getWaitlistId: function(response){
                return response.dfn + ";" + response.clinicName + ";" + response.daysOnList;
            },

            getServiceConnectedCode: function(){
                return this.get('eligibilityCodeExternal') || "NO";
            }
        });

        WaitListEntryEWL.singleInstance = null;

        WaitListEntryEWL.instance = function (attributes) {
            if (attributes) {
                WaitListEntryEWL.singleInstance = new WaitListEntryEWL(attributes);
            } else {
                WaitListEntryEWL.singleInstance = WaitListEntryEWL.singleInstance || new WaitListEntryEWL();
            }
            return WaitListEntryEWL.singleInstance;
        };

        WaitListEntryEWL.clearInstance = function () {
            WaitListEntryEWL.singleInstance = null;
        };

        return WaitListEntryEWL;
    }
);